user/dev discussion of public-inbox itself
 help / color / mirror / code / Atom feed
Search results ordered by [date|relevance]  view[summary|nested|Atom feed]
thread overview below | download mbox.gz: |
* [PATCH 78/82] imap: misc cleanups and notes
  2020-06-10  7:03  7% [PATCH 00/82] public-inbox-imapd: read-only IMAP server Eric Wong
@ 2020-06-10  7:05  6% ` Eric Wong
  0 siblings, 0 replies; 2+ results
From: Eric Wong @ 2020-06-10  7:05 UTC (permalink / raw)
  To: meta

Note some of our limitations for potential hackers.

We'll be renaming "UID_BLOCK" to "UID_SLICE", since "block" is
overused term and "slice" isn't used in our codebase.  Also,
document how "slice" and "epochs" are similar concepts for
different clients.
---
 lib/PublicInbox/IMAP.pm  | 38 +++++++++++++++++++++++++-------------
 lib/PublicInbox/IMAPD.pm |  2 +-
 2 files changed, 26 insertions(+), 14 deletions(-)

diff --git a/lib/PublicInbox/IMAPD.pm b/lib/PublicInbox/IMAPD.pm
index e4dc90e8d0e..fe50de0f7cd 100644
--- a/lib/PublicInbox/IMAPD.pm
+++ b/lib/PublicInbox/IMAPD.pm
@@ -44,7 +44,7 @@ sub imapd_refresh_ibx { # pi_config->each_inbox cb
 	# this case is a 32-bit representation of the creation
 	# date/time of the mailbox"
 	defined($ibx->{uidvalidity} = $mm->created_at) or return;
-	PublicInbox::IMAP::ensure_ranges_exist($imapd, $ibx, $mm->max // 0);
+	PublicInbox::IMAP::ensure_slices_exist($imapd, $ibx, $mm->max // 0);
 
 	# preload to avoid fragmentation:
 	$ibx->description;

^ permalink raw reply related	[relevance 6%]

* [PATCH 00/82] public-inbox-imapd: read-only IMAP server
@ 2020-06-10  7:03  7% Eric Wong
  2020-06-10  7:05  6% ` [PATCH 78/82] imap: misc cleanups and notes Eric Wong
  0 siblings, 1 reply; 2+ results
From: Eric Wong @ 2020-06-10  7:03 UTC (permalink / raw)
  To: meta

So I finally wrote my first IMAP server!  And I'm actually
fairly satisfied with how it's turning out to support a bunch
of other performance + scalability work I've wanted to do.

Some previous notes here:
  https://public-inbox.org/meta/20200609113442.GA16856@dcvr/

I finally seem to have gotten it to play nicely with mutt header
caching, so it's fit for public consumption :)

imaps://news.public-inbox.org/INBOX.comp.mail.public-inbox.meta.0

You can use any username+password, and AUTH=ANONYMOUS also
works if your client does that.

It doesn't support UTF-7 (mailbox names) or advertise UTF-8
in CAPABILITIES, yet; I still have RFCs to read :P

And there's a bunch of new things which could use some
testing from non-mutt/mbsync/offlineimap users.
Maybe you'll find some client-side bugs like I did :P

v1 reindexing also gets a little bit of parallelism :)

Anyways, I'll probably be porting some of the scalability
and slow-storage work to older parts of the code before
fiddling with more IMAP extensions.

Eric Wong (82):
  doc: add some IMAP standards
  nntpd: restrict allowed newsgroup names
  preliminary imap server implementation
  inboxidle: new class to detect inbox changes
  imap: support IDLE
  msgmap: split ->max into its own method
  imap: delay InboxIdle start, support refresh
  imap: implement STATUS command
  imap: use Text::ParseWords::parse_line to handle quoted words
  imap: support LIST command
  t/imapd: support FakeInotify and KQNotify
  imap: support fetch for BODYSTRUCTURE and BODY
  eml: each_part: single part $idx is 1
  imap: allow fetch of partial of BODY[...] and headers
  imap: always include `resp-text' in responses
  imap: split out unit tests and benchmarks
  imap: fix multi-message partial header fetches
  imap: simplify partial fetch structure
  imap: support sequence number FETCH
  imap: do not include ".PEEK" in responses
  imap: support the CLOSE command
  imap: speed up HEADER.FIELDS[.NOT] range fetches
  git: async: flatten the inflight array
  git: do our own read buffering for cat-file
  imap: use git-cat-file asynchronously
  git: idle rbuf for async
  imap: support LSUB command
  imap: FETCH: support comma-delimited ranges
  add imapd compression test
  testcommon: tcp_(server|connect): BAIL_OUT on failure
  *deflate: drop invalid comment about rbuf
  imap: fix pipelining with async git
  git: cat_async: provide requested OID + "missing" on missing blobs
  git: move async_cat reference to PublicInbox::Git
  git: async: automatic retry on alternates change
  imapclient: wrapper for Mail::IMAPClient
  xt: add imapd-validate and imapd-mbsync-oimap
  imap: support out-of-bounds ranges
  xt/perf-imap-list: time refresh_inboxlist
  imap: case-insensitive mailbox name comparisons
  imap: break giant inboxes into sub-inboxes of 50K messages
  imap: start introducing iterative config reloading
  imap: require ".$UID_MIN-$UID_END" suffix
  imapd: ensure LIST is sorted alphabetically, for now
  imap: omit $UID_END from mailbox name, use index
  t/config.t: always compare against git bool behavior
  xt/*: show some tunable parameters
  imap: STATUS and LIST are case-insensitive, too
  imap: EXAMINE/STATUS: return correct counts
  imap: avoid uninitialized warnings on incomplete commands
  imap: start parsing out queries for SQLite and Xapian
  imap: SEARCH: clamp results to the 50K UID range
  imap: allow UID range search on timestamps
  over: get_art: use dbh->prepare_cached
  search: index byte size of a message for IMAP search
  search: index UID for IMAP search, too
  imap: remove dummies from sequence number FETCH
  imap: compile UID FETCH to opcodes
  imap: UID FETCH: optimize for smsg-only case
  imap: UID FETCH: optimize (UID FLAGS) harder
  imap: IDLE: avoid extraneous wakeups, keep-alive
  imap: 30 minute auto-logout timer
  imap: split ->logged_in attribute into a separate class
  searchidx: v1 (re)-index uses git asynchronously
  index: account for CRLF conversion when storing bytes
  imap: rely on smsg->{bytes} for RFC822.SIZE
  imap: UID FETCH requires at least one data item
  imap: LIST shows "INBOX" in all caps
  imap: support 8000 octet lines
  imap: reinstate some message sequence number support
  imap: cleanup ->{uid_base} usage
  imap: FETCH: more granular CRLF conversion
  imap: further speed up HEADER.FIELDS FETCH requests
  imap: FETCH: try to make fake MSNs sequentially
  imap: STATUS/EXAMINE: rely on SQLite overview
  imap: UID SEARCH: support multiple ranges
  imap: wire up Xapian search, msn SEARCH and multiple ranges
  imap: misc cleanups and notes
  imapd: don't bother sorting LIST output
  imap: drop non-UID SEARCH for now
  over: uid_range: remove LIMIT
  imap: FETCH: proper MSN => UID mapping for requests

 Documentation/public-inbox-imapd.pod |   91 ++
 Documentation/standards.perl         |   10 +
 MANIFEST                             |   18 +
 lib/PublicInbox/Config.pm            |   18 +
 lib/PublicInbox/Daemon.pm            |   24 +-
 lib/PublicInbox/DummyInbox.pm        |   22 +
 lib/PublicInbox/Eml.pm               |    9 +-
 lib/PublicInbox/FakeInotify.pm       |   59 ++
 lib/PublicInbox/Git.pm               |  163 +--
 lib/PublicInbox/GitAsyncCat.pm       |   51 +
 lib/PublicInbox/IMAP.pm              | 1397 ++++++++++++++++++++++++++
 lib/PublicInbox/IMAPClient.pm        |  119 +++
 lib/PublicInbox/IMAPD.pm             |  114 +++
 lib/PublicInbox/IMAPdeflate.pm       |  126 +++
 lib/PublicInbox/Import.pm            |    2 +-
 lib/PublicInbox/In2Tie.pm            |   17 +
 lib/PublicInbox/Inbox.pm             |   33 +-
 lib/PublicInbox/InboxIdle.pm         |   79 ++
 lib/PublicInbox/KQNotify.pm          |   66 ++
 lib/PublicInbox/Lock.pm              |    7 +
 lib/PublicInbox/MsgIter.pm           |    2 +-
 lib/PublicInbox/Msgmap.pm            |   20 +-
 lib/PublicInbox/NNTPD.pm             |   12 +-
 lib/PublicInbox/NNTPdeflate.pm       |    1 -
 lib/PublicInbox/Over.pm              |   50 +-
 lib/PublicInbox/Search.pm            |   32 +-
 lib/PublicInbox/SearchIdx.pm         |   89 +-
 lib/PublicInbox/SearchIdxShard.pm    |   11 +-
 lib/PublicInbox/Smsg.pm              |    8 +-
 lib/PublicInbox/TestCommon.pm        |    7 +-
 lib/PublicInbox/V2Writable.pm        |   10 +-
 script/public-inbox-imapd            |   14 +
 t/config.t                           |   15 +-
 t/eml.t                              |    2 +-
 t/git.t                              |   40 +-
 t/imap.t                             |  133 +++
 t/imapd-tls.t                        |  204 ++++
 t/imapd.t                            |  398 ++++++++
 t/import.t                           |    5 +-
 t/inbox_idle.t                       |   72 ++
 t/nntpd.t                            |    5 +-
 t/over.t                             |    3 +
 t/search.t                           |   19 +
 xt/cmp-msgstr.t                      |    1 -
 xt/cmp-msgview.t                     |    1 -
 xt/eml_check_limits.t                |    6 +-
 xt/git_async_cmp.t                   |    2 +-
 xt/imapd-mbsync-oimap.t              |  132 +++
 xt/imapd-validate.t                  |  177 ++++
 xt/mem-msgview.t                     |    1 +
 xt/msgtime_cmp.t                     |    1 -
 xt/perf-msgview.t                    |    1 -
 52 files changed, 3718 insertions(+), 181 deletions(-)
 create mode 100644 Documentation/public-inbox-imapd.pod
 create mode 100644 lib/PublicInbox/DummyInbox.pm
 create mode 100644 lib/PublicInbox/FakeInotify.pm
 create mode 100644 lib/PublicInbox/GitAsyncCat.pm
 create mode 100644 lib/PublicInbox/IMAP.pm
 create mode 100644 lib/PublicInbox/IMAPClient.pm
 create mode 100644 lib/PublicInbox/IMAPD.pm
 create mode 100644 lib/PublicInbox/IMAPdeflate.pm
 create mode 100644 lib/PublicInbox/In2Tie.pm
 create mode 100644 lib/PublicInbox/InboxIdle.pm
 create mode 100644 lib/PublicInbox/KQNotify.pm
 create mode 100644 script/public-inbox-imapd
 create mode 100644 t/imap.t
 create mode 100644 t/imapd-tls.t
 create mode 100644 t/imapd.t
 create mode 100644 t/inbox_idle.t
 create mode 100644 xt/imapd-mbsync-oimap.t
 create mode 100644 xt/imapd-validate.t

^ permalink raw reply	[relevance 7%]

Results 1-2 of 2 | reverse | options above
-- pct% links below jump to the message on this page, permalinks otherwise --
2020-06-10  7:03  7% [PATCH 00/82] public-inbox-imapd: read-only IMAP server Eric Wong
2020-06-10  7:05  6% ` [PATCH 78/82] imap: misc cleanups and notes Eric Wong

Code repositories for project(s) associated with this public inbox

	https://80x24.org/public-inbox.git

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).
diff --git a/lib/PublicInbox/IMAP.pm b/lib/PublicInbox/IMAP.pm
index 67bc32ab59b..1ba3a3ff1bb 100644
--- a/lib/PublicInbox/IMAP.pm
+++ b/lib/PublicInbox/IMAP.pm
@@ -7,7 +7,16 @@
 # slow storage.
 #
 # data notes:
-# * NNTP article numbers are UIDs
+#
+# * NNTP article numbers are UIDs, mm->created_at is UIDVALIDITY
+#
+# * public-inboxes are sliced into mailboxes of 50K messages
+#   to not overload MUAs: $NEWSGROUP_NAME.$SLICE_INDEX
+#   Slices are similar in concept to v2 "epochs".  Epochs
+#   are for the limitations of git clients, while slices are
+#   for the limitations of IMAP clients.
+#
+# * sequence numbers are estimated based on slice
 
 package PublicInbox::IMAP;
 use strict;
@@ -37,7 +46,7 @@ die "neither Email::Address::XS nor Mail::Address loaded: $@" if !$Address;
 sub LINE_MAX () { 8000 } # RFC 2683 3.2.1.5
 
 # changing this will cause grief for clients which cache
-sub UID_BLOCK () { 50_000 }
+sub UID_SLICE () { 50_000 }
 
 # these values area also used for sorting
 sub NEED_SMSG () { 1 }
@@ -187,7 +196,7 @@ sub on_inbox_unlock {
 	my ($self, $ibx) = @_;
 	my $new = $ibx->over->max;
 	my $uid_base = $self->{uid_base} // 0;
-	my $uid_end = $uid_base + UID_BLOCK;
+	my $uid_end = $uid_base + UID_SLICE;
 	defined(my $old = $self->{-idle_max}) or die 'BUG: -idle_max unset';
 	$new = $uid_end if $new > $uid_end;
 	if ($new > $old) {
@@ -226,7 +235,7 @@ sub cmd_idle ($$) {
 	my $ibx = $self->{ibx} or return "$tag BAD no mailbox selected\r\n";
 	$self->{-idle_tag} = $tag;
 	my $max = $ibx->over->max;
-	my $uid_end = $self->{uid_base} + UID_BLOCK;
+	my $uid_end = $self->{uid_base} + UID_SLICE;
 	my $sock = $self->{sock} or return;
 	my $fd = fileno($sock);
 	# only do inotify on most recent slice
@@ -260,12 +269,12 @@ sub cmd_done ($$) {
 	"$idle_tag OK Idle done\r\n";
 }
 
-sub ensure_ranges_exist ($$$) {
+sub ensure_slices_exist ($$$) {
 	my ($imapd, $ibx, $max) = @_;
 	defined(my $mb_top = $ibx->{newsgroup}) or return;
 	my $mailboxes = $imapd->{mailboxes};
 	my @created;
-	for (my $i = int($max/UID_BLOCK); $i >= 0; --$i) {
+	for (my $i = int($max/UID_SLICE); $i >= 0; --$i) {
 		my $sub_mailbox = "$mb_top.$i";
 		last if exists $mailboxes->{$sub_mailbox};
 		$mailboxes->{$sub_mailbox} = $ibx;
@@ -281,14 +290,14 @@ sub inbox_lookup ($$) {
 	my ($self, $mailbox) = @_;
 	my ($ibx, $exists, $uidnext, $uid_base);
 	if ($mailbox =~ /\A(.+)\.([0-9]+)\z/) {
 		# old mail: inbox.comp.foo.$uid_block_idx
 		my $mb_top = $1;
-		$uid_base = $2 * UID_BLOCK;
+		$uid_base = $2 * UID_SLICE;
 		$ibx = $self->{imapd}->{mailboxes}->{lc $mailbox} or return;
 		my $max;
 		($exists, $uidnext, $max) = $ibx->over->imap_status($uid_base,
-							$uid_base + UID_BLOCK);
-		ensure_ranges_exist($self->{imapd}, $ibx, $max);
+							$uid_base + UID_SLICE);
+		ensure_slices_exist($self->{imapd}, $ibx, $max);
 	} else { # check for dummy inboxes
 		$mailbox = lc $mailbox;
 		$ibx = $self->{imapd}->{mailboxes}->{$mailbox} or return;
@@ -296,7 +305,7 @@ sub inbox_lookup ($$) {
 		# if "INBOX.foo.bar" is selected and "INBOX.foo.bar.0",
 		# check for new UID ranges (e.g. "INBOX.foo.bar.1")
 		if (my $z = $self->{imapd}->{mailboxes}->{"$mailbox.0"}) {
-			ensure_ranges_exist($self->{imapd}, $z, $z->over->max);
+			ensure_slices_exist($self->{imapd}, $z, $z->over->max);
 		}
 
 		$uid_base = $exists = 0;
@@ -580,7 +589,7 @@ sub op_crlf_bdy { ${$_[4]->{bdy}} =~ s/(?<!\r)\n/\r\n/sg if $_[4]->{bdy} }
 sub uid_clamp ($$$) {
 	my ($self, $beg, $end) = @_;
 	my $uid_min = $self->{uid_base} + 1;
-	my $uid_end = $uid_min + UID_BLOCK - 1;
+	my $uid_end = $uid_min + UID_SLICE - 1;
 	$$beg = $uid_min if $$beg < $uid_min;
 	$$end = $uid_end if $$end > $uid_end;
 }
@@ -595,7 +604,7 @@ sub range_step ($$) {
 		$$range_csv = undef;
 	}
 	my $uid_base = $self->{uid_base};
-	my $uid_end = $uid_base + UID_BLOCK;
+	my $uid_end = $uid_base + UID_SLICE;
 	if ($range =~ /\A([0-9]+):([0-9]+)\z/) {
 		($beg, $end) = ($1 + 0, $2 + 0);
 		uid_clamp($self, \$beg, \$end);
@@ -1206,6 +1215,9 @@ sub args_ok ($$) { # duplicated from PublicInbox::NNTP
 # returns 1 if we can continue, 0 if not due to buffered writes or disconnect
 sub process_line ($$) {
 	my ($self, $l) = @_;
+
+	# TODO: IMAP allows literals for big requests to upload messages
+	# (which we don't support) but maybe some big search queries use it.
 	my ($tag, $req, @args) = parse_line('[ \t]+', 0, $l);
 	pop(@args) if (@args && !defined($args[-1]));
 	if (@args && uc($req) eq 'UID') {
@@ -281,14 +290,14 @@ sub inbox_lookup ($$) {
 	my ($ibx, $exists, $uidnext, $uid_base);
 	if ($mailbox =~ /\A(.+)\.([0-9]+)\z/) {
-		# old mail: inbox.comp.foo.$uid_block_idx
+		# old mail: inbox.comp.foo.$SLICE_IDX
 		my $mb_top = $1;